/* C interface to Bohren & Huffman Mie code */

#ifndef BHMIE_H
#define BHMIE_H 1

#ifdef __cplusplus
extern "C" {
#endif

  /* Maximum number of angles that can be reported */
#define BHMIE_MXNANG 1000

  /* Interface to Fortran code */
  void bhmie_(double* x, double* refind, int* nang,
	      double* s1, double* s2,
	      double* qext, double* qsca, double* qback, double* g);

  /* Single Mie calculation for electrical size x, refractive index
     (refind_r + i*refind_i), returning extinction, scattering and
     backscatter efficiencies qext, qsca and qback, respectively, and
     asymmetry factor g */
  inline
  void
  mie(/* Inputs: */
      float x, float refind_r, float refind_i,
      /* Outputs */
      float* qext, float* qsca, float* qback, float* g)
  {
    int nang = 2;
    double s1_dummy[2*(2*nang-1)];
    double s2_dummy[2*(2*nang-1)];
    double refind[2] = {refind_r, refind_i};
    double x_in = x;
    double qext_out, qsca_out, qback_out, g_out;
    bhmie_(&x_in, refind, &nang, s1_dummy, s2_dummy, 
	   &qext_out, &qsca_out, &qback_out, &g_out);
    *qext = qext_out;
    *qsca = qsca_out;
    *qback = qback_out;
    *g = g_out;
  }
  
  /* This version also returns the phase function at 2*nang-1 angles
     between 0 and 180 degrees */
  inline
  void
  mie_phase_function(/* Inputs: */
		     float x, float refind_r, float refind_i,
		     int nang,
		     /* Outputs */
		     float* qext, float* qsca, float* qback, float* g,
		     float* pp)
  {
    if (nang < 2) { nang = 2; }
    if (nang > BHMIE_MXNANG) { nang = BHMIE_MXNANG; }
    int max = 2*nang-1;
    double s1[2*max];
    double s2[2*max];
    double x_in = x;
    double refind[2] = {refind_r, refind_i};
    double qext_out, qsca_out, qback_out, g_out;
    bhmie_(&x_in, refind, &nang, s1, s2, 
	   &qext_out, &qsca_out, &qback_out, &g_out);
    int i;
    *qext = qext_out;
    *qsca = qsca_out;
    *qback = qback_out;
    *g = g_out;
    float factor = 1.0/(*qsca * x * x);
    for (i = 0; i < max; i++) {
      pp[i] = (s1[i*2]*s1[i*2]+s1[i*2+1]*s1[i*2+1]
	       + s2[i*2]*s2[i*2]+s2[i*2+1]*s2[i*2+1]) * factor;
    }
  }
  
#ifdef __cplusplus
}
#endif

#endif
